function model = team_training_code(input_directory,output_directory) % train_PCG_classifier
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Purpose: Train PCG classifiers and obtain the models
% Inputs:
% 1. input_directory
% 2. output_directory
%
% Outputs:
% 1. model: trained model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    use_gt_segmentation = false;

    % Find text files
    patient_files=dir(fullfile(input_directory,'*.txt'));
    patient_files={patient_files.name};
    patient_files=sort(patient_files); % To help debugging
%     global patient_breath;
%     patient_files = patient_files(~patient_breath);
    num_patient_files=length(patient_files);
    
    disp("["+datestr(now,'HH:MM:SS')+"]:")
    fprintf('Loading data for %d patients...\n', num_patient_files)
    
    % Extract classes from data
    classes_murmur={};
    classes_outcome={};
    for j=1:num_patient_files
        current_class_murmur=get_class_murmur(fullfile(input_directory,patient_files{j}));
        classes_murmur=unique([classes_murmur current_class_murmur]);
    
        current_class_outcome=get_class_outcome(fullfile(input_directory,patient_files{j}));
        classes_outcome=unique([classes_outcome current_class_outcome]);
    end
    
    classes_murmur=sort(classes_murmur);
    num_classes_murmur=length(classes_murmur);

    classes_outcome=sort(classes_outcome);
    num_classes_outcome=length(classes_outcome);

    disp("["+datestr(now,'HH:MM:SS')+"]: Training HSMM...")
    HSMM.B = [];
    HSMM.pi_v = [];
    HSMM.obs = [];
    if ~use_gt_segmentation
        addpath("HSMM");
        [B, pi_v, obs] = train_segmentation(input_directory);
        HSMM.B = B;
        HSMM.pi_v = pi_v;
        HSMM.obs = obs;
    end
    disp("["+datestr(now,'HH:MM:SS')+"]: Done.")
    
    % Extracting features and labels
    disp("["+datestr(now,'HH:MM:SS')+"]: Extracting features and labels...")
    
    features=[];
    labels_murmur=categorical;
    labels_outcome=categorical;
    
    for j=1:num_patient_files
        fprintf('%d/%d \n',j,num_patient_files)
    
        current_header=get_header(fullfile(input_directory,patient_files{j}));
        current_recordings=load_recordings(input_directory,current_header);
        current_segments = [];

        if use_gt_segmentation
            current_segments = load_segmentation(input_directory,current_header);
        end
        
        current_features=get_features(current_header, current_recordings, current_segments, HSMM, true);
        features(j,:) = current_features(:);
    
        labels_murmur(j)=get_class_murmur(fullfile(input_directory,patient_files{j}));
        labels_outcome(j)=get_class_outcome(fullfile(input_directory,patient_files{j}));
    end
    save(strcat(output_directory,'/features.mat'),'features');
    save(strcat(output_directory,'/labels.mat'),'labels_murmur','labels_outcome');
    save(strcat(output_directory,'/HSMM.mat'),'HSMM');
    rng(12);
%%
%     [model_murmur,coef_murmur] = train_and_validate_SVM_murmur(features,labels_murmur,0.9);
%     [model_outcome,coef_outcome] = train_and_validate_SVM_outcome(features,labels_outcome,0.9);

    model_murmur = train_murmur_classifier(features,labels_murmur);
    model_outcome = train_outcome_classifier(features,labels_outcome);

%     [model_murmur,coef_murmur] = train_and_validate_RF_murmur(features,labels_murmur,0.9);
%     [model_outcome,coef_outcome] = train_and_validate_RF_outcome(features,labels_outcome,0.9);
    save_model(model_murmur,classes_murmur,[],model_outcome,classes_outcome,[],HSMM,output_directory);
    
    disp('Done.')

end

function save_model(model_m,classes_m,coef_m,model_o,classes_o,coef_o,HSMM,output_directory) %save_PCG_model
    % Save results.
    filename = fullfile(output_directory,'model.mat');
    save(filename,'model_m','classes_m','coef_m','model_o','classes_o','coef_o','HSMM','-v7.3');
    
    disp("["+datestr(now,'HH:MM:SS')+"]: Done.")
end

function class=get_class_murmur(input_header)
    current_header=get_header(input_header);
    
    class=current_header(startsWith(current_header,'#Murmur'));
    class=strsplit(class{1},':');
    class=strtrim(class{2});
end

function current_header=get_header(input_header)
    current_header=fileread(input_header);
    current_header=strsplit(current_header,'\n');
end

function current_recordings=load_recordings(input_directory,current_header)
    recording_files=get_recording_files(current_header);
    
    current_recordings={};
    
    for j=1:length(recording_files)
        file = fullfile(input_directory,strtrim(recording_files{j}));
%         disp(file);
        current_recordings{j}=audioread(file);
    end
end

function current_segments=load_segmentation(input_directory,current_header)
    recording_files=get_segmentation_files(current_header);
    
    current_segments={};
    
    for j=1:length(recording_files)
        file = fullfile(input_directory,strtrim(recording_files{j}));
        annot = readmatrix(file,'FileType','text','Delimiter','\t');
        starts = annot(:,1);
        ends = annot(:,2);
        Fs = 1000;
        
        s1_s = round(starts(annot(:,3)==1)*Fs)+1;
        s1_e = round(ends(annot(:,3)==1)*Fs);
        s2_s = round(starts(annot(:,3)==3)*Fs)+1;
        s2_e = round(ends(annot(:,3)==3)*Fs);
        sys_s = round(starts(annot(:,3)==2)*Fs)+1;
        sys_e = round(ends(annot(:,3)==2)*Fs);
        dia_s = round(starts(annot(:,3)==4)*Fs)+1;
        dia_e = round(ends(annot(:,3)==4)*Fs);

        current_segments{j}={[s1_s,s1_e],[s2_s,s2_e],[sys_s,sys_e],[dia_s,dia_e]};
    end
end

function recording_files=get_recording_files(current_header)
    recording_files={};
    
    num_locations=strsplit(current_header{1},' ');
    num_locations=str2double(num_locations{2});
    
    for j=2:num_locations+1
        current_line=strsplit(current_header{j},' ');
        recording_files{j-1}=current_line{3};
    end
end
function segmentation_files=get_segmentation_files(current_header)
    segmentation_files={};
    
    num_locations=strsplit(current_header{1},' ');
    num_locations=str2double(num_locations{2});
    
    for j=2:num_locations+1
        current_line=strsplit(current_header{j},' ');
        segmentation_files{j-1}=current_line{4};
    end
end
function class=get_class_outcome(input_header)
    current_header=get_header(input_header);
    
    class=current_header(startsWith(current_header,'#Outcome'));
    class=strsplit(class{1},':');
    class=strtrim(class{2});
end